/*!
 * \file main.c
 * \brief Mdulo Timer/PWM - PWM.
 * \mainpage Projeto de Controle da Cor de um Led via uma Botoeira.
 * \details Este c&oacute;digo ilustra a configura&ccedil;&atilde;o dos canais do mdulo TPM para 
 *          variar a intensidade dos leds RGB (PWM) de forma a ter uma maior gama de cores. 
 * 			(Leitura recomendada: Se&ccedil;&otilde;es 31.4.6 e 31.4.7 de KL25 Sub-Family Reference Manual
 * 			 Se&ccedil;&atilde;o 12.3 de Kinetis L Peripheral Module Quick Reference)
 * \author Wu Shin-Ting
 * \date 08/05/2016
 * \note Documenta&ccedil;&atilde;o do c&oacute;digo no formato de Doxygen:
 *       http://mcuoneclipse.com/2014/09/01/automatic-documentation-generation-doxygen-with-processor-expert/
 * \attention Observem bem que muitas macros s&atilde;o simples c&oacute;digos bin&aacute;rios. Ao
 * fazer opera&ccedil;&atilde;o l&oacute;gica OR entre elas e o conte&uacute;do dos registradore, 
 * os bits em '1' s&atilde;o mantidos. Para garantir que n&atilde;o haja efeitos colaterais,
 * recomenda-se que zere primeiro os bits do campo a ser alterado antes de uma opera&ccedil;&atilde;o OR.
 */

#include <stdlib.h>
#include "estrutura.h"
#include "port.h"
#include "ledRGB.h"
#include "lcdled.h"
#include "tpm.h"
#include "nvic.h"
#include "atrasos.h"

/*! 
 * vari&aacute;vel de controle da contagem de tempo (m&uacute;ltiplo de 0.25s)
 */
uint32_t tempo;       

/*!
 * vari&aacute;vel de cor selecionada
 */         
uint8_t indice;            

/*!
 * vari&aacute;vel de estado da botoeira PTA12 (1: pressionado; 0 solto)
 */
uint8_t estado_pta12;       

/*!
 * varivel de estado do ledRGB
 */
uint8_t apagado;

/*!
 * vetor de cores do led
 */
cor cores_estaticas[] = {{ACESO,APAGADO,APAGADO},{ACESO,ACESO,ACESO},{ACESO,ACESO,APAGADO},
				{APAGADO,ACESO,ACESO},{ACESO,APAGADO,ACESO},{APAGADO,APAGADO,APAGADO}};

/*!
 * buffers de dados de interface com UART
 */
buffer_circular buffer_receptor, buffer_transmissor;

#define COUNTER_OVF xxxx         ///< valor m&aacute;ximo do TPMx_CNT

int main(void)
{	
	/*!
	 * Cores adaptadas a partir de https://pymolwiki.org/index.php/Color_Values
	 */
    cor_pf cores[13] = {
    		{1.0, 0.0, 0.0},       ///< vermelho
    		{1.0, 0.42, 0.42},     ///< salmao 
    		{1.0, 1.0, 0.15},      ///< amarelo claro
    	    {0.0, 1.0, 0.0},       ///< verde
    	    {0.05, 0.9, 0.05},     ///< verde floresta 
    	    {0., 0.75, 0.75},      ///< azul berilo    
    	    {0.02, 0.3, 1.0},      ///< azul celeste 
    	    {0.0, 0.0, 1.0},       ///< azul           
    		{0.6, 0.1, 0.6},       ///< violeta        
    	    {0.65, 0.37, 0.27},    ///< marrom         
    	    {0.698, 0.35, 0.40},   ///< tijolo         
    		{1.0, 1.0, 1.0},        ///< branco
    		{0.0, 0.0, 0.0}        ///< preto
    };
    
    /*!
     * Inicializa as botoeiras para serem controladas por TPM
     */
    initPort(A);
	setaMux(ePORTA_PCR4, xxxx);           ///< GPIO
	setaMux(ePORTA_PCR12, xxxx);			 ///< TPM1_C0
    
    /*! 
     * Inicializa os tr&ecirc;s pinos dos leds para serem controlados por TPM
     */
	initPort(B);                          ///< leds R e G
	initPort(D);                          ///< led B
	setaMux(ePORTB_PCR18, xxxx);           ///< TPM2_C0
	setaMux(ePORTB_PCR19, xxxx);			  ///< TPM2_C1
	setaMux(ePORTD_PCR1, xxxx);			  ///< TPM0_C1
	
	/*!
	 * Inicializa LedLCD apagando os leds vermelhos. Clock da porta C  habilitado
	 */
	initLcdledGPIO();
	enviaLed(xxxx);
	
	/*!
	 * Altera a fun&ccedil;&atilde;o do segundo led vermelho do shield FEEC para ser controlado por TPM
	 */
	setaMux(ePORTC_PCR1, 0x4);           ///< TPM0_C0
	
	/*!
	 * Configura TPMx
	 * Per&iacute;odo = 1ms
	 * Pr&eacute;-escala=0b110;
	 * Contagem M&aacute;xima=COUNTER_OVF
	 * Contagem crescente
	 */
	initTPM(0,0b110,(uint16_t)(COUNTER_OVF),0);  
	initTPM(1,0b110,(uint16_t)(COUNTER_OVF),0);  
	initTPM(2,0b110,(uint16_t)(COUNTER_OVF),0);
	
	/*! 
	 * Inicializa canal 0 com fun&ccedil;&atilde;o Output Compare no TPM0
	 * Reseta o sinal quando "casam os valores" (match)
	 */ 
	initTPM0Cn(0,xxxx,TPM0_CNT+5);  
		
	/*! 
	 * Inicializa canal 1 como desabilitados no TPM0
	 */ 
	initTPM0Cn(1,xxxx,(unsigned short)(1.1*COUNTER_OVF));  /*! led azul */
	
	/*! 
	 * Inicializa canais 0,1 como desabilitados no TPM2
	 */ 
    initTPM2Cn(0,xxxx,(unsigned short)(1.1*COUNTER_OVF));                                /*! led vermelho */
    initTPM2Cn(1,xxxx,(unsigned short)(1.1*COUNTER_OVF));                                /*! led verde */
    
    /*!
     * Inicializa canal 0 com fun&ccedil;&atilde;o Input Capture no TPM1
     * Captura em ambas as bordas
     */
	initTPM1Cn(0,xxxx,0);   

	/*!
	 * Ativa interrup&ccedil;&atilde;o do TPM0_C0 no NVIC com nvel de prioridade 3
	 */
	enableNVICTPM0(xxxx);

	/*!
	 * Habilita interrup&ccedil;&atilde;o do TPM1_C0 com nvel de prioridade 2
	 */
    enableNVICTPM1(xxxx);
	enableTPM1CnInterrup(0);

	/*!
	 * Inicializa o estado
	 */
	indice = 12;
	
	for(;;) {
		/*!  Recarga dos contadores dos canais PWM */
		if(indice > 12) indice = 12;
		
		TPM2_SC |= 	TPM_SC_CMOD(0x0);           ///< desabilita clock LPTPM (Se&ccedil;&atilde;o 31.4.8.2 em ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/KL25P80M48SF0RM.pdf)
		TPM0_SC |= 	TPM_SC_CMOD(0x0);           ///< desabilita clock LPTPM (Se&ccedil;&atilde;o 31.4.8.2 em ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/KL25P80M48SF0RM.pdf)
		TPM2_C0V = TPM_CnV_VAL((unsigned short)(cores[indice].vermelho*COUNTER_OVF));     ///< % R
		TPM2_C1V = TPM_CnV_VAL((unsigned short)(cores[indice].verde*COUNTER_OVF));        ///< % G		
		TPM0_C1V = TPM_CnV_VAL((unsigned short)(cores[indice].azul*COUNTER_OVF));         ///< % B
		TPM2_SC |= 	TPM_SC_CMOD(0x1);           ///< habilita clock LPTPM (Se&ccedil;&atilde;o 31.4.8.2 em ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/KL25P80M48SF0RM.pdf)
		TPM0_SC |= 	TPM_SC_CMOD(0x1);           ///< habilita clock LPTPM (Se&ccedil;&atilde;o 31.4.8.2 em ftp://ftp.dca.fee.unicamp.br/pub/docs/ea871/ARM/KL25P80M48SF0RM.pdf)
		
		/*! Sele&ccedil;&atilde;o de modo, transi&ccedil;&atilde;o e n&iacute;vel (Modo: PWM alinhado a borda) */ 
		/*! Led R */
		if (cores[indice].vermelho < 0.001)
			TPM2_C0SC &= ~(TPM_CnSC_MSB_MASK |      	///< MSB=TPM2_C0SC[5]=0
						TPM_CnSC_MSA_MASK |         	///< MSA=TPM2_C0SC[4]=0
						TPM_CnSC_ELSB_MASK |        	///< ELSB=TPM2_C0SC[3]=0
						TPM_CnSC_ELSA_MASK );       	///< ELSA=TPM2_C0SC[2]=0
		else {
			TPM2_C0SC |= (TPM_CnSC_MSB_MASK |         	///< MSB=TPM2_C1SC[5]=1
					TPM_CnSC_ELSB_MASK |          	///< ELSB=TPM2_C1SC[3]=1
					TPM_CnSC_ELSA_MASK );         	///< ELSA=TPM2_C1SC[2]=1
			TPM2_C0SC &= ~TPM_CnSC_MSA_MASK;          	///< MSA=TPM2_C1SC[4]=0
		}
		/*! Led G */
		if (cores[indice].verde < 0.001)
			TPM2_C1SC &= ~(TPM_CnSC_MSB_MASK |      	///< MSB=TPM2_C1SC[5]=0
						TPM_CnSC_MSA_MASK |         	///< MSA=TPM2_C1SC[4]=0
						TPM_CnSC_ELSB_MASK |        	///< ELSB=TPM2_C1SC[3]=0
						TPM_CnSC_ELSA_MASK );       	///< ELSA=TPM2_C1SC[2]=0
		else {
			TPM2_C1SC |= (TPM_CnSC_MSB_MASK |       	///< MSB=TPM2_C1SC[5]=1
					TPM_CnSC_ELSB_MASK |        	///< ELSB=TPM2_C1SC[3]=1
					TPM_CnSC_ELSA_MASK );       	///< ELSA=TPM2_C1SC[2]=1
			TPM2_C1SC &= ~TPM_CnSC_MSA_MASK;        	///< MSA=TPM2_C1SC[4]=0
		}
		/*! Led B */
		if (cores[indice].azul < 0.001)		
			TPM0_C1SC &= ~(TPM_CnSC_MSB_MASK |      ///< MSB=TPM0_C1SC[5]=0
						TPM_CnSC_MSA_MASK |         ///< MSA=TPM0_C1SC[4]=0
						TPM_CnSC_ELSB_MASK |        ///< ELSB=TPM0_C1SC[3]=0
						TPM_CnSC_ELSA_MASK );       ///< ELSA=TPM0_C1SC[2]=0
		else {
			TPM0_C1SC |= (TPM_CnSC_MSB_MASK |         	///< MSB=TPM2_C1SC[5]=1
					TPM_CnSC_ELSB_MASK |        	///< ELSB=TPM2_C1SC[3]=1
					TPM_CnSC_ELSA_MASK );       	///< ELSA=TPM2_C1SC[2]=1
			TPM0_C1SC &= ~TPM_CnSC_MSA_MASK;         	///< MSA=TPM2_C1SC[4]=0
		}
		
		delay10us(100000);
		
		/*! Sele&ccedil;&atilde;o de modo, transi&ccedil;&atilde;o e n&iacute;vel (desabilita os 3 canais) */ 
		TPM2_C0SC &= ~(TPM_CnSC_MSB_MASK |      	///< MSB=TPM2_C0SC[5]=0
					TPM_CnSC_MSA_MASK |         	///< MSA=TPM2_C0SC[4]=0
					TPM_CnSC_ELSB_MASK |        	///< ELSB=TPM2_C0SC[3]=0
					TPM_CnSC_ELSA_MASK );       	///< ELSA=TPM2_C0SC[2]=0
		TPM2_C1SC &= ~(TPM_CnSC_MSB_MASK |      	///< MSB=TPM2_C1SC[5]=0
					TPM_CnSC_MSA_MASK |         	///< MSA=TPM2_C1SC[4]=0
					TPM_CnSC_ELSB_MASK |        	///< ELSB=TPM2_C1SC[3]=0
					TPM_CnSC_ELSA_MASK );       	///< ELSA=TPM2_C1SC[2]=0
		TPM0_C1SC &= ~(TPM_CnSC_MSB_MASK |      ///< MSB=TPM0_C1SC[5]=0
					TPM_CnSC_MSA_MASK |         ///< MSA=TPM0_C1SC[4]=0
					TPM_CnSC_ELSB_MASK |        ///< ELSB=TPM0_C1SC[3]=0
					TPM_CnSC_ELSA_MASK );       ///< ELSA=TPM0_C1SC[2]=0
				
		delay10us(100000);
	}
	
}
